import 'package:openweather/openweather.dart';
import "dart:convert";
import 'package:test/test.dart';
import './api_key.dart';

const SAMPLE_TRIGGER = """
{
   "time_period":{
      "start":{
         "expression":"after",
         "amount":132000000
      },
      "end":{
         "expression":"after",
         "amount":132000000
      }
   },
   "conditions":[
      {
         "name":"temp",
         "expression":"\$lt",
         "amount":270
      }
   ],
   "area":[
      {
         "type":"Point",
         "coordinates":[
            23.71,
            90.41
         ]
      }
   ]
}
""";

void main() {
  group('Test Current Weather API', () {
    OpenWeatherClient client;

    setUp(() {
      client = new OpenWeatherClient(API_KEY);
    });

    test('Test Current Weather by City Name', () async {
      CityWeather weather = await client.getCityWeatherByCityName("Dhaka");
      expect(weather.name, "Dhaka");
      expect(weather.timezone, 19800);
    });
    test('Test Current Weather by City Name with Country Code', () async {
      CityWeather weather =
          await client.getCityWeatherByCityName("Dhaka", "bd");
      expect(weather.name, "Dhaka");
      expect(weather.timezone, 21600);
    });
    test('Test Current Weather by City ID', () async {
      CityWeather weather = await client.getCityWeatherByCityID(1185241);
      expect(weather.name, "Dhaka");
      expect(weather.timezone, 21600);
    });
    test('Test Current Weather by Coordinate', () async {
      CityWeather weather =
          await client.getCityWeatherByCoordinate(lat: 23.71, lon: 90.41);
      expect(weather.name, "Dhaka");
      expect(weather.timezone, 21600);
    });
    test('Test Current Weather by ZIPCode', () async {
      CityWeather weather = await client.getCityWeatherByZIPCode(90001);
      expect(weather.name, "Los Angeles");
      expect(weather.timezone, -25200);
    });
    test('Test Current Weather by ZIPCode with country', () async {
      CityWeather weather = await client.getCityWeatherByZIPCode(1212, 'bd');
      expect(weather.name, "Gulshan Model Town");
      expect(weather.timezone, 21600);
    });

    test('Test Current Weather from cities in a rectangle', () async {
      RectLocResponse weatherData =
          await client.getCityWeatherByRectangle(12, 32, 15, 37, 10);
      expect(weatherData.cod.toString(), "200");
    });
    test('Test Weathers in cities in a cycle', () async {
      CitiesInCycleResponse weatherData =
          await client.getCitiesWeatherInCycle(23.71, 90.41, 10);
      expect(weatherData.list.length, 10);
      expect(weatherData.list[0].name, "Dhaka");
      expect(weatherData.cod.toString(), "200");
    });
    test('Test get Cities Weather by IDs', () async {
      CitiesByIDResponse weatherData =
          await client.getCitiesWeatherbyID([524901, 703448, 2643743]);
      expect(weatherData.list.length, 3);
      expect(weatherData.cnt, 3);
    });
  });

  group('Test 3 Hourly Weather API', () {
    OpenWeatherClient client;

    setUp(() {
      client = new OpenWeatherClient(API_KEY);
    });

    test('Test 3 Hourly Forecast by City Name', () async {
      ForecastResponse weather = await client.getForecastByCityName("Dhaka");
      expect(weather.cnt, 40);
      expect(weather.cod, 200);
      expect(weather.list.length, 40);
    });
    test('Test 3 Hourly Forecast by City Name with Country Code', () async {
      ForecastResponse weather =
          await client.getForecastByCityName("Dhaka", "bd");
      expect(weather.cnt, 40);
      expect(weather.cod, 200);
      expect(weather.list.length, 40);
    });
    test('Test 3 Hourly Forecast by City ID', () async {
      ForecastResponse weather = await client.getForecastByCityID(1185241);
      expect(weather.cnt, 40);
      expect(weather.cod, 200);
      expect(weather.list.length, 40);
    });
    test('Test 3 Hourly Forecast by Coordinate', () async {
      ForecastResponse weather =
          await client.getForecastByCoordinates(lat: 23.71, lon: 90.41);
      expect(weather.cnt, 40);
      expect(weather.cod, 200);
      expect(weather.list.length, 40);
    });
    test('Test 3 Hourly Forecast by ZIPCode', () async {
      ForecastResponse weather = await client.getForecastByZipcode(90001);
      expect(weather.cnt, 40);
      expect(weather.cod, 200);
      expect(weather.list.length, 40);
    });
    test('Test 3 Hourly Forecast by ZIPCode with country', () async {
      ForecastResponse weather = await client.getForecastByZipcode(1212, 'bd');
      expect(weather.cnt, 40);
      expect(weather.cod, 200);
      expect(weather.list.length, 40);
    });
  });

  group("UVIndex Tests", () {
    OpenWeatherClient client;

    setUp(() {
      client = new OpenWeatherClient(API_KEY);
    });

    test('Test Current UVIndex by Coordinate', () async {
      UVIndex uidx =
          await client.getCurrentUVIndexByCoordinates(lat: 23.71, lon: 90.41);
      expect(uidx.value is double, true);
    });

    test('Test UVIndex Forecast by Coordinate', () async {
      List<UVIndex> uidxs = await client.getUVIndexForecastByCoordinates(
          lat: 23.71, lon: 90.41, cnt: 8);
      expect(uidxs[0].value is double, true);
      expect(uidxs.length, 8);
    });

    test('Test UVIndex History by Coordinate', () async {
      DateTime endDate = new DateTime.now();
      DateTime startDate = endDate.subtract(Duration(days: 5));
      List<UVIndex> uidxs = await client.getUVIndexHistoryByCoordinates(
          lat: 23.71, lon: 90.41, startDate: startDate, endDate: endDate);
      expect(uidxs[0].value is double, true);
      expect(
          uidxs[0].date, OpenWeatherClient.generateMiddayTimestamp(startDate));
      expect(uidxs.length, 6);
    });
  });

  group("Trigger Test", () {
    OpenWeatherClient client;

    setUp(() {
      client = OpenWeatherClient(API_KEY);
    });

    test("Test Post Trigger from JSON Data", () async {
      var wa = WeatherAlert.fromJSON(jsonDecode(SAMPLE_TRIGGER));
      WeatherAlert res = await client.postTrigger(wa.serialized);
      expect(res.id is String, true);
      client.deleteTrigger(res.id);
    });

    test("Test Post Trigger from WeatherAlert", () async {
      var wa = WeatherAlert.fromJSON(jsonDecode(SAMPLE_TRIGGER));
      WeatherAlert res = await client.postTriggerByWeatherAlert(wa);
      expect(res.id is String, true);
      client.deleteTrigger(res.id);
    });

    test("Test Update Trigger from JSON Data", () async {
      var war = WeatherAlert.fromJSON(jsonDecode(SAMPLE_TRIGGER));
      WeatherAlert wa = await client.postTriggerByWeatherAlert(war);
      wa.area = [
        new Area(
            type: "Point",
            coordinates:
                Point(coordinate: Coordinate(latitude: 45, longitude: 90)))
      ];
      WeatherAlert res = await client.updateTrigger(wa.id, wa.serialized);
      expect(res.id is String, true);
      var sdata = res.area[0].coordinates.serialized;
      expect(sdata[0], 45);
      expect(sdata[1], 90);
      client.deleteTrigger(res.id);
    });

    test("Test Update Trigger from WeatherAlert", () async {
      var war = WeatherAlert.fromJSON(jsonDecode(SAMPLE_TRIGGER));
      WeatherAlert wa = await client.postTriggerByWeatherAlert(war);
      wa.area = [
        new Area(
            type: "Point",
            coordinates:
                Point(coordinate: Coordinate(latitude: 35, longitude: 40)))
      ];
      WeatherAlert res = await client.updateTriggerByWeatherAlert(wa);
      expect(res.id is String, true);
      var sdata = res.area[0].coordinates.serialized;
      expect(sdata[0], 35);
      expect(sdata[1], 40);
      client.deleteTrigger(res.id);
    });
  });
}
